# coding: utf-8

from django.conf.urls.defaults import *

urlpatterns = patterns('',
    
    # BOOKMARKS
    url(r'^bookmark/add/$', 'grappelli_safe.views.bookmarks.add_bookmark', name="grp_bookmark_add"),
    url(r'^bookmark/remove/$', 'grappelli_safe.views.bookmarks.remove_bookmark', name="grp_bookmark_remove"),
    url(r'^bookmark/get/$', 'grappelli_safe.views.bookmarks.get_bookmark', name="grp_bookmark_get"),
    
    # HELP
    url(r'^help/(?P<object_id>\d+)/$', 'grappelli_safe.views.help.detail', name="grp_help_detail"),
    url(r'^help', 'grappelli_safe.views.help.help', name="grp_help"),
    
    # GENERIC
    url(r'^obj_lookup/$', 'grappelli_safe.views.generic.generic_lookup', name="grp_generic_lookup"),
    
    # FOREIGNKEY LOOKUP
    url(r'^related_lookup/$', 'grappelli_safe.views.related.related_lookup', name="grp_related_lookup"),
    url(r'^m2m_lookup/$', 'grappelli_safe.views.related.m2m_lookup', name="grp_m2m_lookup")
    
    # AUTOCOMPLETE LOOKUP
    #url(r'^autocomplete_lookup/$', 'grappelli_safe.views.related.autocomplete_lookup', name="grp_autocomplete_lookup"),
    #url(r'^autocomplete_lookup_id/$', 'grappelli_safe.views.related.autocomplete_lookup_id', name="grp_autocomplete_lookup_id"),
    
    # M2M AUTOCOMPLETE LOOKUP
    #url(r'^m2m_autocomplete_lookup/$', 'grappelli_safe.views.related.m2m_autocomplete_lookup', name="grp_m2m_autocomplete_lookup"),
    #url(r'^m2m_autocomplete_lookup_id/$', 'grappelli_safe.views.related.m2m_autocomplete_lookup_id', name="grp_m2m_autocomplete_lookup_id"),
    
)


# coding: utf-8

from django.db import models, transaction
from django.utils.translation import ugettext as _

from grappelli_safe.fields import PositionField

class Help(models.Model):
    """
    Help Entry.
    """
    
    title = models.CharField(_('Title'), max_length=50)
    
    # order
    order = PositionField(_('Order'))
    
    class Meta:
        app_label = "grappelli_safe"
        verbose_name = _('Help')
        verbose_name_plural = _('Help')
        ordering = ['order']
    
    def __unicode__(self):
        return u"%s" % (self.title)
    
    save = transaction.commit_on_success(models.Model.save)
    

class HelpItem(models.Model):
    """
    Help Entry Item.
    """
    
    help = models.ForeignKey(Help, verbose_name=_('Help'))
    title = models.CharField(_('Title'), max_length=200)
    link = models.CharField(_('Link'), max_length=200, help_text=_('The Link should be relative, e.g. /admin/blog/.'))
    body = models.TextField(_('Body'))
    
    # order
    order = PositionField(_('Order'), unique_for_field='help')
    
    class Meta:
        app_label = "grappelli_safe"
        verbose_name = _('Help Entry')
        verbose_name_plural = _('Help Entries')
        ordering = ['help', 'order']
    
    def __unicode__(self):
        return u"%s" % (self.title)
    
    def get_body(self):
        body = self.body.replace('<h2>', '</div><div class="module"><h2>')
        return body
    
    save = transaction.commit_on_success(models.Model.save)
    




# coding: utf-8

from django.http import HttpResponse
from django.db import models

def related_lookup(request):
    
    if request.method == 'GET':
        if request.GET.has_key('object_id') and request.GET.has_key('app_label') and request.GET.has_key('model_name'):
            object_id = request.GET.get('object_id')
            app_label = request.GET.get('app_label')
            model_name = request.GET.get('model_name')
            if object_id:
                try:
                    model = models.get_model(app_label, model_name)
                    obj = model.objects.get(pk=object_id)
                    obj_text = unicode(obj)
                except:
                    obj_text = "Not Found"
            else:
                obj_text = ""
        else:
            obj_text = "Error"
    else:
        obj_text = "Error"
    
    return HttpResponse(obj_text, mimetype='text/plain; charset=utf-8')
    

def m2m_lookup(request):
    obj_text = []
    if request.method == 'GET':
        if request.GET.has_key('object_id') and request.GET.has_key('app_label') and request.GET.has_key('model_name'):
            object_ids = request.GET.get('object_id').split(',')
            app_label = request.GET.get('app_label')
            model_name = request.GET.get('model_name')
            
            for obj_id in object_ids:
                try:
                    model = models.get_model(app_label, model_name)
                    obj = model.objects.get(pk=obj_id)
                    obj_text.append(unicode(obj))
                except:
                    obj_text.append("Not Found")
        else:
            obj_text.append("Error")
    else:
        obj_text.append("Error")
    obj_text = ", ".join(obj_text)
    
    return HttpResponse(obj_text, mimetype='text/plain; charset=utf-8')



# coding: utf-8

from django.db import models, transaction
from django.utils.translation import ugettext as _

from grappelli_safe.fields import PositionField

ITEM_CATEGORY_CHOICES = (
    ('1', 'internal'),
    ('2', 'external'),
)

class Navigation(models.Model):
    """
    Sidebar-Navigation on the Admin Index-Site.
    """
    
    title = models.CharField(_('Title'), max_length=30)
    
    # order
    order = PositionField(_('Order'))
    
    class Meta:
        app_label = "grappelli_safe"
        verbose_name = _('Navigation')
        verbose_name_plural = _('Navigation')
        ordering = ['order',]
    
    def __unicode__(self):
        return u"%s" % (self.title)
        
    save = transaction.commit_on_success(models.Model.save)
    

class NavigationItem(models.Model):
    """
    Navigation Item.
    """
    
    navigation = models.ForeignKey(Navigation, verbose_name=_('Navigation'))
    title = models.CharField(_('Title'), max_length=30)
    link = models.CharField(_('Link'), max_length=200, help_text=_('The Link should be relative, e.g. /admin/blog/.'))
    category = models.CharField(_('Category'), max_length=1, choices=ITEM_CATEGORY_CHOICES)
    
    # users
    users = models.ManyToManyField('auth.User', limit_choices_to={'is_staff': True}, verbose_name=_('Users'), blank=True, related_name="admin_navigation_users")
    groups = models.ManyToManyField('auth.Group', verbose_name=_('Groups'), blank=True, related_name="admin_navigation_groups")
    
    # order
    order = PositionField(_('Order'), unique_for_field='navigation')
    
    class Meta:
        app_label = "grappelli_safe"
        verbose_name = _('Navigation Item')
        verbose_name_plural = _('Navigation Items')
        ordering = ['navigation', 'order']
    
    def __unicode__(self):
        return u"%s" % (self.title)
        
    save = transaction.commit_on_success(models.Model.save)
    




# coding: utf-8

from django.db import connection, models
from django.db.models.signals import post_delete, post_save


qn = connection.ops.quote_name


class PositionField(models.IntegerField):
    """A model field to manage the position of an item within a collection.

    By default all instances of a model are treated as one collection; if the
    ``unique_for_field`` argument is used, each value of the specified field is
    treated as a distinct collection.

    ``PositionField`` values work like list indices, including the handling of
    negative values.  A value of ``-2`` will be cause the position to be set to
    the second to last position in the collection.  The implementation differs
    from standard list indices in that values that are too large or too small
    are converted to the maximum or minimum allowed value respectively.

    When the value of a ``PositionField`` in a model instance is modified, the
    positions of other instances in the same collection are automatically
    updated to reflect the change.

    Assigning a value of ``None`` to a ``PositionField`` will cause the instance
    to be moved to the end of the collection (or appended to the collection, in
    the case of a new instance).

    """
    def __init__(self, verbose_name=None, name=None, unique_for_field=None,
                 *args, **kwargs):
        # blank values are used to move an instance to the last position
        kwargs.setdefault('blank', True)

        # unique constraints break the ability to execute a single query to
        # increment or decrement a set of positions; they also require the use
        # of temporary placeholder positions which result in undesirable
        # additional queries
        unique = kwargs.get('unique', False)
        if unique:
            raise TypeError(
                '%s cannot have a unique constraint' % self.__class__.__name__
            )
        # TODO: raise exception if position field appears in unique_together

        super(PositionField, self).__init__(verbose_name, name, *args, **kwargs)
        self.unique_for_field = unique_for_field

    def contribute_to_class(self, cls, name):
        super(PositionField, self).contribute_to_class(cls, name)

        # use this object as the descriptor for field access
        setattr(cls, self.name, self)

        # adjust related positions in response to a delete or save
        post_delete.connect(self._on_delete, sender=cls)
        post_save.connect(self._on_save, sender=cls)

    def get_internal_type(self):
        # all values will be positive after pre_save
        return 'PositiveIntegerField'

    def pre_save(self, model_instance, add):
        current, updated = self._get_instance_cache(model_instance)
        
        # existing instance, position not modified; no cleanup required
        if current is not None and updated is None:
            self._reset_instance_cache(model_instance, current)
            return current

        count = self._get_instance_peers(model_instance).count()
        if current is None:
            max_position = count
        else:
            max_position = count - 1
        min_position = 0

        # new instance; appended; no cleanup required
        if current is None and (updated == -1 or updated >= max_position):
            self._reset_instance_cache(model_instance, max_position)
            return max_position

        if max_position >= updated >= min_position:
            # positive position; valid index
            position = updated
        elif updated > max_position:
            # positive position; invalid index
            position = max_position
        elif abs(updated) <= (max_position + 1):
            # negative position; valid index

            # add 1 to max_position to make this behave like a negative list
            # index.  -1 means the last position, not the last position minus 1

            position = max_position + 1 + updated
        else:
            # negative position; invalid index
            position = min_position

        # instance inserted; cleanup required on post_save
        self._set_instance_cache(model_instance, position)
        return position

    def __get__(self, instance, owner):
        if instance is None:
            raise AttributeError('%s must be accessed via instance' % self.name)
        current, updated = self._get_instance_cache(instance)
        if updated is None:
            return current
        return updated

    def __set__(self, instance, value):
        if instance is None:
            raise AttributeError('%s must be accessed via instance' % self.name)
        self._set_instance_cache(instance, value)

    def _get_instance_cache(self, instance):
        try:
            current, updated = getattr(instance, self.get_cache_name())
        except (AttributeError, TypeError):
            current, updated = None, None
        return current, updated

    def _reset_instance_cache(self, instance, value):
        try:
            delattr(instance, self.get_cache_name())
        except AttributeError:
            pass
        setattr(instance, self.get_cache_name(), (value, None))

    def _set_instance_cache(self, instance, value):
        # TODO: Find a more robust way to determine if instance exists in
        # the db; this is necessary because the position field won't be
        # initialized to None when models.Manager.create is called with an
        # explicit position.

        # FIXME: This breaks pre-newforms-admin when setting a position gt the
        # maximum possible position -- something about how models are
        # instantiated keeps this method from doing the right thing.
        # Fortunately it works on newforms-admin, so it will be moot soon.

        has_pk = bool(getattr(instance, instance._meta.pk.attname))

        # default to None for existing instances; -1 for new instances
        #updated = None if has_pk else -1
        if has_pk:
            updated = None
        else:
            updated = -1

        try:
            current = getattr(instance, self.get_cache_name())[0]
        except (AttributeError, TypeError):
            if has_pk:
                current = value
            else:
                current = None
                if value is not None:
                    updated = value
        else:
            if value is None:
                updated = -1
            elif value != current:
                updated = value

        setattr(instance, self.get_cache_name(), (current, updated))

    def _get_instance_peers(self, instance):
        # return a queryset containing all instances of the model that belong
        # to the same collection as instance; either all instances of a model
        # or all instances with the same value in unique_for_field
        filters = {}
        if self.unique_for_field:
            unique_for_field = instance._meta.get_field(self.unique_for_field)
            unique_for_value = getattr(instance, unique_for_field.attname)
            if unique_for_field.null and unique_for_value is None:
                filters['%s__isnull' % unique_for_field.name] = True
            else:
                filters[unique_for_field.name] = unique_for_value
        return instance.__class__._default_manager.filter(**filters)

    def _on_delete(self, sender, instance, **kwargs):
        current, updated = self._get_instance_cache(instance)
        
        # decrement positions gt current
        operations = [self._get_operation_sql('-')]
        conditions = [self._get_condition_sql('>', current)]

        cursor = connection.cursor()
        cursor.execute(self._get_update_sql(instance, operations, conditions))
        self._reset_instance_cache(instance, None)

    def _on_save(self, sender, instance, **kwargs):
        current, updated = self._get_instance_cache(instance)

        # no cleanup required
        if updated is None:
            return None

        if current is None:
            # increment positions gte updated
            operations = [self._get_operation_sql('+')]
            conditions = [self._get_condition_sql('>=', updated)]
        elif updated > current:
            # decrement positions gt current and lte updated
            operations = [self._get_operation_sql('-')]
            conditions = [
                self._get_condition_sql('>', current),
                self._get_condition_sql('<=', updated)
            ]
        else:
            # increment positions lt current and gte updated
            operations = [self._get_operation_sql('+')]
            conditions = [
                self._get_condition_sql('<', current),
                self._get_condition_sql('>=', updated)
            ]

        # exclude instance from the update
        conditions.append('%(pk_field)s != %(pk)s' % {
            'pk': getattr(instance, instance._meta.pk.attname),
            'pk_field': qn(instance._meta.pk.column)
        })

        cursor = connection.cursor()
        cursor.execute(self._get_update_sql(instance, operations, conditions))
        self._reset_instance_cache(instance, updated)

    def _get_update_sql(self, instance, operations=None, conditions=None):
        operations = operations or []
        conditions = conditions or []

        # TODO: add params and operations to update auto_now date(time)s

        params = {
            'position_field': qn(self.column),
            'table': qn(instance._meta.db_table),
        }
        if self.unique_for_field:
            unique_for_field = instance._meta.get_field(self.unique_for_field)
            unique_for_value = getattr(instance, unique_for_field.attname)

            params['unique_for_field'] = qn(unique_for_field.column)

            # this field is likely to be indexed; put it first
            if unique_for_field.null and unique_for_value is None:
                conditions.insert(0, '%(unique_for_field)s IS NULL')
            else:
                params['unique_for_value'] = unique_for_value
                conditions.insert(0,
                                  '%(unique_for_field)s = %(unique_for_value)s')

        query = 'UPDATE %(table)s'
        query += ' SET %s' % ', '.join(operations)
        query += ' WHERE %s' % ' AND '.join(conditions)

        return query % params

    def _get_condition_sql(self, gt_or_lt, position):
        return '%%(position_field)s %(gt_or_lt)s %(position)s' % {
            'gt_or_lt': gt_or_lt,
            'position': position
        }

    def _get_operation_sql(self, plus_or_minus):
        return """
        %%(position_field)s = (%%(position_field)s %(plus_or_minus)s 1)""" % {
            'plus_or_minus': plus_or_minus
        }
        





# coding: utf-8

from django.conf import settings

# Admin Site Title
ADMIN_HEADLINE = getattr(settings, "GRAPPELLI_ADMIN_HEADLINE", 'Grappelli')
ADMIN_TITLE = getattr(settings, "GRAPPELLI_ADMIN_TITLE", 'Grappelli')

# Link to your Main Admin Site (no slashes at start and end)
ADMIN_URL = getattr(settings, "GRAPPELLI_ADMIN_URL", '/admin/')


# coding: utf-8

from django.db import models, transaction
from django.utils.translation import ugettext as _

from grappelli_safe.fields import PositionField

class Bookmark(models.Model):
    """
    Bookmark.
    """
    
    user = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True}, verbose_name=_('User'), related_name="admin_bookmark_set")
    
    class Meta:
        app_label = "grappelli_safe"
        verbose_name = _('Bookmark')
        verbose_name_plural = _('Bookmarks')
        ordering = ['user',]
    
    def __unicode__(self):
        return u"%s" % (self.user)
    
    save = transaction.commit_on_success(models.Model.save)
    

class BookmarkItem(models.Model):
    """
    Bookmark Item.
    """
    
    bookmark = models.ForeignKey(Bookmark, verbose_name=_('Bookmark'))
    title = models.CharField(_('Title'), max_length=80)
    link = models.CharField(_('Link'), max_length=200, help_text=_('The Link should be relative, e.g. /admin/blog/.'))
    
    # order
    order = PositionField(_('Order'), unique_for_field='bookmark')
    
    class Meta:
        app_label = "grappelli_safe"
        verbose_name = _('Bookmark Item')
        verbose_name_plural = _('Bookmark Items')
        ordering = ['order']
    
    def __unicode__(self):
        return u"%s" % (self.title)
    
    save = transaction.commit_on_success(models.Model.save)
    




# -*- coding: utf-8 -*-

# imports
import urllib

# django imports
from django.shortcuts import HttpResponse, render_to_response
from django.http import HttpResponseRedirect
from django.contrib.admin.views.decorators import staff_member_required
from django.utils.translation import ugettext as _
from django.template import RequestContext

# grappelli imports
from grappelli_safe.models.bookmarks import Bookmark, BookmarkItem
from grappelli_safe.settings import ADMIN_TITLE, ADMIN_URL
try:
    # django SVN
    from django.views.decorators.csrf import csrf_exempt
except:
    # django 1.1
    from django.contrib.csrf.middleware import csrf_exempt

@csrf_exempt
def add_bookmark(request):
    """
    Add Site to Bookmarks.
    """

    if request.method == 'POST':
        if request.POST.get('path') and request.POST.get('title'):
            next = urllib.unquote(request.POST.get('path'))
            try:
                bookmark = Bookmark.objects.get(user=request.user)
            except Bookmark.DoesNotExist:
                bookmark = Bookmark(user=request.user)
                bookmark.save()
            try:
                bookmarkitem = BookmarkItem.objects.get(bookmark=bookmark, link=urllib.unquote(request.POST.get('path')))
                msg = ['error', _('Site is already bookmarked.')]
            except BookmarkItem.DoesNotExist:
                try:
                    bookmarkitem = BookmarkItem(bookmark=bookmark, title=request.POST.get('title'), link=urllib.unquote(request.POST.get('path')))
                    bookmarkitem.save()
                    msg = ['success', _('Site was added to Bookmarks.')]
                except:
                    msg = ['error', _('Site could not be added to Bookmarks.')]
        else:
            msg = ['error', _('Site could not be added to Bookmarks.')]
            next = request.POST.get('path')
    else:
        msg = ['error', _('Site could not be added to Bookmarks.')]
        next = ADMIN_URL

    # MESSAGE & REDIRECT
    if not request.session.get('grappelli'):
        request.session['grappelli'] = {}
    request.session['grappelli']['message'] = msg
    request.session.modified = True
    return HttpResponseRedirect(next)
add_bookmark = staff_member_required(add_bookmark)


def remove_bookmark(request):
    """
    Remove Site from Bookmarks.
    """

    if request.GET:
        if request.GET.get('path'):
            next = urllib.unquote(request.GET.get('path'))
            try:
                bookmarkitem = BookmarkItem.objects.get(bookmark__user=request.user, link=urllib.unquote(request.GET.get('path')))
                bookmarkitem.delete()
                msg = ['success', _('Site was removed from Bookmarks.')]
            except BookmarkItem.DoesNotExist:
                msg = ['error', _('Site could not be removed from Bookmarks.')]
        else:
            msg = ['error', _('Site could not be removed from Bookmarks.')]
            next = ADMIN_URL
    else:
        msg = ['error', _('Site could not be removed from Bookmarks.')]

    # MESSAGE & REDIRECT
    if not request.session.get('grappelli'):
        request.session['grappelli'] = {}
    request.session['grappelli']['message'] = msg
    request.session.modified = True
    return HttpResponseRedirect(next)
remove_bookmark = staff_member_required(remove_bookmark)


def get_bookmark(request):
    """
    Get Bookmarks for the currently logged-in User (AJAX request).
    """

    if request.method == 'GET':
        if request.GET.get('path'):
            object_list = BookmarkItem.objects.filter(bookmark__user=request.user).order_by('order')
            #print urllib.unquote(request.GET.get('path'))
            try:
                bookmark = Bookmark.objects.get(user=request.user)
            except Bookmark.DoesNotExist:
                bookmark = Bookmark(user=request.user)
                bookmark.save()
            try:
                BookmarkItem.objects.get(bookmark__user=request.user, link=urllib.unquote(request.GET.get('path')))
                is_bookmark = True
            except BookmarkItem.DoesNotExist:
                is_bookmark = False
        else:
            object_list = ""
            is_bookmark = ""
    else:
        object_list = ""
        is_bookmark = ""

    return render_to_response('admin/includes_grappelli/bookmarks.html', RequestContext(request, {
        'object_list': object_list,
        'bookmark': bookmark,
        'is_bookmark': is_bookmark,
        'admin_title': ADMIN_TITLE,
        'path': request.GET.get('path', ''),
    }))
get_bookmark = staff_member_required(get_bookmark)




# coding: utf-8

from django.shortcuts import render_to_response
from django.template import RequestContext
from django.shortcuts import get_object_or_404
from django.contrib.admin.views.decorators import staff_member_required
from django.utils.translation import ugettext as _

from grappelli_safe.models.help import Help, HelpItem

def detail(request, object_id):
    """
    CMS Help (Detail).
    """
    
    obj = get_object_or_404(HelpItem, pk=object_id)
    menu = Help.objects.filter(helpitem__isnull=False).distinct()
    
    return render_to_response('grappelli/help/help_detail.html', {
        'object': obj,
        'menu': menu,
        'title': obj.title,
    }, context_instance=RequestContext(request) )
detail = staff_member_required(detail)

def help(request):
    """
    CMS Help (Overview).
    """
    
    menu = Help.objects.filter(helpitem__isnull=False).distinct()
    
    return render_to_response('grappelli/help/help.html', {
        'menu': menu,
        'title': _('Help'),
    }, context_instance=RequestContext(request) )
help = staff_member_required(help)



# coding: utf-8

from django.contrib import admin
from django.utils.translation import ugettext as _
from django import forms
from django.conf import settings

from grappelli_safe.models.navigation import Navigation, NavigationItem
from grappelli_safe.models.bookmarks import Bookmark, BookmarkItem
from grappelli_safe.models.help import Help, HelpItem


class NavigationItemInline(admin.StackedInline):
    
    model = NavigationItem
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        ('', {
            'fields': ('title', 'link', 'category',)
        }),
        ('', {
            'fields': ('groups', 'users',),
        }),
        ('', {
            'fields': ('order',),
        }),
    )
    filter_horizontal = ('users',)
    
    # Grappelli Options
    allow_add = True


class NavigationOptions(admin.ModelAdmin):
    
    # List Options
    list_display = ('order', 'title',)
    list_display_links = ('title',)
    
    # Fieldsets
    fieldsets = (
        ('', {
            'fields': ('title', 'order',)
        }),
    )
    
    # Misc
    save_as = True
    
    # Inlines
    inlines = [NavigationItemInline]
    
    # Grappelli Options
    order = 0


class BookmarkItemInline(admin.TabularInline):
    
    model = BookmarkItem
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        ('', {
            'fields': ('title', 'link', 'order',)
        }),
    )
    
    # Grappelli Options
    allow_add = True


class BookmarkOptions(admin.ModelAdmin):
    
    # List Options
    list_display = ('user',)
    list_display_links = ('user',)
    
    # Fieldsets
    fieldsets = (
        ('', {
            'fields': ('user',)
        }),
    )
    
    # Misc
    save_as = True
    
    # Inlines
    inlines = [BookmarkItemInline]
    
    # Grappelli Options
    order = 1
    
    def has_change_permission(self, request, obj=None):
        has_class_permission = super(BookmarkOptions, self).has_change_permission(request, obj)
        if not has_class_permission:
            return False
        if obj is not None and not request.user.is_superuser and request.user.id != obj.user.id:
            return False
        return True
    
    def save_model(self, request, obj, form, change):
        if not request.user.is_superuser:
            if not change:
                obj.user = request.user
        obj.save()
    
    def queryset(self, request):
        if request.user.is_superuser:
            return Bookmark.objects.all()
        return Bookmark.objects.filter(user=request.user)


class HelpItemInline(admin.StackedInline):
    
    model = HelpItem
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        ('', {
            'fields': ('title', 'link', 'body', 'order',)
        }),
    )
    
    # Grappelli Options
    allow_add = True


class HelpOptions(admin.ModelAdmin):
    
    # List Options
    list_display = ('order', 'title',)
    list_display_links = ('title',)
    
    # Fieldsets
    fieldsets = (
        ('', {
            'fields': ('title', 'order',)
        }),
    )
    
    # Misc
    save_as = True
    
    # Inlines
    inlines = [HelpItemInline]
    
    # Grappelli Options
    order = 2
    
    # Media
    class Media:
        js = [
            settings.ADMIN_MEDIA_PREFIX + 'tinymce/jscripts/tiny_mce/tiny_mce.js',
            settings.ADMIN_MEDIA_PREFIX + 'tinymce_setup/tinymce_setup.js',
        ]


class HelpItemOptions(admin.ModelAdmin):
    
    # List Options
    list_display = ('order', 'title',)
    list_display_links = ('title',)
    
    # Fieldsets
    fieldsets = (
        ('', {
            'fields': ('help', 'title', 'link', 'body', 'order',)
        }),
    )
    
    # Grappelli Options
    order = 3
    
    # Media
    class Media:
        js = [
            settings.ADMIN_MEDIA_PREFIX + 'tinymce/jscripts/tiny_mce/tiny_mce.js',
            settings.ADMIN_MEDIA_PREFIX + 'tinymce_setup/tinymce_setup.js',
        ]


#admin.site.register(Navigation, NavigationOptions)
#admin.site.register(Bookmark, BookmarkOptions)
#admin.site.register(Help, HelpOptions)
#admin.site.register(HelpItem, HelpItemOptions)





# coding: utf-8

# imports
import re

# django imports
from django import template
from django.contrib.contenttypes.models import ContentType
from django.contrib import admin
from django.db import models
from django.db.models import Q
from django.contrib.auth.models import Group

# grappelli imports
from grappelli_safe.models.help import HelpItem
from grappelli_safe.models.navigation import Navigation, NavigationItem
from grappelli_safe.settings import *

register = template.Library()


# GENERIC OBJECTS
class do_get_generic_objects(template.Node):
    
    def __init__(self):
        pass
    
    def render(self, context):
        return_string = "var MODEL_URL_ARRAY = {"
        for c in ContentType.objects.all().order_by('id'):
            return_string = "%s%d: '%s/%s'," % (return_string, c.id, c.app_label, c.model)
        return_string = "%s}" % return_string[:-1]
        return return_string

def get_generic_relation_list(parser, token):
    """
    Returns a list of installed applications and models.
    Needed for lookup of generic relationships.
    """
    
    tokens = token.contents.split()
    return do_get_generic_objects()
    
register.tag('get_generic_relation_list', get_generic_relation_list)


# CONTEXT-SENSITIVE HELP
def get_help(path):
    """
    Context Sensitive Help (currently not implemented).
    """
    
    try:
        helpitem = HelpItem.objects.get(link=path)
    except HelpItem.DoesNotExist:
        helpitem = ""
    
    return { 'helpitem': helpitem }

register.inclusion_tag('admin/includes_grappelli/help.html')(get_help)


# NAVIGATION
def get_navigation(user):
    """
    User-related Navigation/Sidebar on the Admin Index Page.
    """
    
    if user.is_superuser:
        object_list = NavigationItem.objects.all()
    else:
        object_list = NavigationItem.objects.filter(Q(groups__in=user.groups.all()) | Q(users=user)).distinct()
    
    return { 'object_list': object_list }
    
register.inclusion_tag('admin/includes_grappelli/navigation.html')(get_navigation)


# SEARCH FIELDS VERBOSE
class GetSearchFields(template.Node):
    
    def __init__(self, opts, var_name):
        self.opts = template.Variable(opts)
        self.var_name = var_name
    
    def render(self, context):
        opts = str(self.opts.resolve(context)).split('.')
        model = models.get_model(opts[0], opts[1])
        try:
            field_list = admin.site._registry[model].search_fields_verbose
        except:
            field_list = ""
        
        context[self.var_name] = ", ".join(field_list)
        return ""


def do_get_search_fields_verbose(parser, token):
    """
    Get search_fields_verbose in order to display on the Changelist.
    """
    
    try:
        tag, arg = token.contents.split(None, 1)
    except:
        raise template.TemplateSyntaxError, "%s tag requires arguments" % token.contents.split()[0]
    m = re.search(r'(.*?) as (\w+)', arg)
    if not m:
        raise template.TemplateSyntaxError, "%r tag had invalid arguments" % tag
    opts, var_name = m.groups()
    return GetSearchFields(opts, var_name)

register.tag('get_search_fields_verbose', do_get_search_fields_verbose)


# ADMIN_TITLE
def get_admin_title():
    """
    Returns the Title for the Admin-Interface.
    """
    
    return ADMIN_TITLE
    
register.simple_tag(get_admin_title)


# ADMIN_URL
def get_admin_url():
    """
    Returns the URL for the Admin-Interface.
    """
    
    return ADMIN_URL
    
register.simple_tag(get_admin_url)


# GRAPPELLI MESSAGING SYSTEM
def get_messages(session):
    """
    Get Success and Error Messages.
    """
    
    try:
        msg = session['grappelli']['message']
        del session['grappelli']['message']
        session.modified = True
    except:
        msg = ""
    
    return {
        'message': msg
    }
    
register.inclusion_tag('admin/includes_grappelli/messages.html')(get_messages)





# coding: utf-8

import re
from django import http, template
from django.contrib.admin import ModelAdmin
from django.contrib.admin import actions
from django.contrib.auth import authenticate, login
from django.db.models.base import ModelBase
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.utils.functional import update_wrapper
from django.utils.safestring import mark_safe
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy, ugettext as _
from django.views.decorators.cache import never_cache
from django.conf import settings
try:
    set
except NameError:
    from sets import Set as set     # Python 2.3 fallback
    
from django.contrib.admin.sites import AdminSite

class GrappelliSite(AdminSite):
    """
    An AdminSite object encapsulates an instance of the Django admin application, ready
    to be hooked in to your URLconf. Models are registered with the AdminSite using the
    register() method, and the root() method can then be used as a Django view function
    that presents a full admin interface for the collection of registered models.
    """
    
    def __init__(self, name=None, app_name='admin'):
        self._registry = {} # model_class class -> admin_class instance
        self.root_path = None
        if name is None:
            self.name = 'admin'
        else:
            self.name = name
        self.app_name = app_name
        self._actions = {'delete_selected': actions.delete_selected}
        self._global_actions = self._actions.copy()
        self.groups = {}
        self.collections = {}
        self.group_template = ""
        self.collection_template = ""
    
    def index(self, request, extra_context=None):
        """
        Displays the main admin index page, which lists all of the installed
        apps that have been registered in this site.
        """
        app_dict = {}
        user = request.user
        for model, model_admin in self._registry.items():
            app_label = model._meta.app_label
            has_module_perms = user.has_module_perms(app_label)
            
            if has_module_perms:
                perms = model_admin.get_model_perms(request)
                
                # Check whether user has any perm for this module.
                # If so, add the module to the model_list.
                if True in perms.values():
                    try:
                        order = model_admin.order
                    except:
                        order = 0
                    model_dict = {
                        'order': order,
                        'name': capfirst(model._meta.verbose_name_plural),
                        'admin_url': mark_safe('%s/%s/' % (app_label, model.__name__.lower())),
                        'perms': perms,
                    }
                    if app_label in app_dict:
                        app_dict[app_label]['models'].append(model_dict)
                    else:
                        app_dict[app_label] = {
                            'app_label': app_label,
                            'name': app_label.title(),
                            'app_url': app_label + '/',
                            'has_module_perms': has_module_perms,
                            'models': [model_dict],
                        }
        
        # Sort the apps alphabetically.
        app_list = app_dict.values()
        app_list.sort(lambda x, y: cmp(x['name'], y['name']))
        
        # First: Sort the models alphabetically within each app.
        # Second: Sort the models according to their order-attribute.
        for app in app_list:
            app['models'].sort(lambda x, y: cmp(x['name'], y['name']))
            app['models'].sort(lambda x, y: cmp(x['order'], y['order']))
        
        # set cookie
        if not request.session.get('grappelli'):
            request.session['grappelli'] = {}
        request.session['grappelli']['home'] = request.get_full_path()
        request.session.modified = True
        
        # Assign Apps to Groups
        group_list = {}
        custom_app_list = app_list
        for k,v in self.groups.items():
            if request.GET.get("g") and k != int(request.GET.get("g")):
                continue
            if request.GET.get("c") and k not in self.collections[int(request.GET.get("c"))]['groups']:
                continue
            group_list[k] = v
            application_list = []
            for app in v['apps']:
                try:
                    application_list.append(app_dict[app])
                    # remove assigned app from custom app_list
                    custom_app_list = [d for d in custom_app_list if d.get('app_label') != app]
                except:
                    pass
            if len(application_list):
                group_list[k]['applications'] = application_list
            else:
                group_list[k]['applications'] = ""
        
        # Subsections for Groups and Collections
        # set template and title
        # clear app_list
        if request.GET.get("g"):
            try:
                title = group_list[int(request.GET.get("g"))]['title']
            except:
                title = _('Site administration')
            try:
                group_template = group_list[int(request.GET.get("g"))]['template']
            except:
                group_template = None
            tpl = group_template or self.group_template or "admin/index_group.html"
            custom_app_list = []
        elif request.GET.get("c"):
            try:
                title = self.collections[int(request.GET.get("c"))]['title']
            except:
                title = _('Site administration')
            try:
                collection_template = self.collections[int(request.GET.get("c"))]['template']
            except:
                collection_template = None
            tpl = collection_template or self.collection_template or "admin/index_collection.html"
            custom_app_list = []
        else:
            title = _('Site administration')
            tpl = self.index_template or "admin/index.html"
        
        # Reset grouplist if users has no permissions
        if not app_list:
            group_list = {}
        
        context = {
            'title': title,
            'app_list': custom_app_list,
            'root_path': self.root_path,
            'group_list': group_list
        }
        context.update(extra_context or {})
        context_instance = template.RequestContext(request, current_app=self.name)
        return render_to_response(tpl, context,
            context_instance=context_instance
        )
    index = never_cache(index)
    
    def app_index(self, request, app_label, extra_context=None):
        user = request.user
        has_module_perms = user.has_module_perms(app_label)
        app_dict = {}
        for model, model_admin in self._registry.items():
            if app_label == model._meta.app_label:
                if has_module_perms:
                    perms = model_admin.get_model_perms(request)
                    
                    # Check whether user has any perm for this module.
                    # If so, add the module to the model_list.
                    if True in perms.values():
                        try:
                            order = model_admin.order
                        except:
                            order = 0
                        model_dict = {
                            'order': order,
                            'name': capfirst(model._meta.verbose_name_plural),
                            'admin_url': '%s/' % model.__name__.lower(),
                            'perms': perms,
                        }
                        if app_dict:
                            app_dict['models'].append(model_dict),
                        else:
                            # First time around, now that we know there's
                            # something to display, add in the necessary meta
                            # information.
                            app_dict = {
                                'name': app_label.title(),
                                'app_url': '',
                                'has_module_perms': has_module_perms,
                                'models': [model_dict],
                            }
        if not app_dict:
            raise http.Http404('The requested admin page does not exist.')
        
        # Sort the models alphabetically within each app.
        app_dict['models'].sort(lambda x, y: cmp(x['name'], y['name']))
        app_dict['models'].sort(lambda x, y: cmp(x['order'], y['order']))
        
        context = {
            'title': _('%s administration') % capfirst(app_label),
            'app_list': [app_dict],
            'root_path': self.root_path,
        }
        context.update(extra_context or {})
        context_instance = template.RequestContext(request, current_app=self.name)
        return render_to_response(self.app_index_template or ('admin/%s/app_index.html' % app_label,
            'admin/app_index.html'), context,
            context_instance=context_instance
        )


